/*
 * Hardware initialization.
 *
 * $LastChangedRevision: 483 $
 *
 * Copyright (c) 2013, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
/*
 * This file is licensed according to the BSD 3-clause license as follows:
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
 *       the names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "Target.h"
#include "Main.h"
#include "Hardware.h"

/* private prototypes */
static void sendSerial(tSerial* serial, uint8 data);
static uint8 receiveSerial(tSerial* serial);
static uint8 isReceiveDataAvailable(tSerial* serial);
static uint8 isSendReady(tSerial* serial);

/* private macros */
#define F_FRC           8000000         /* fast RC internal oscillator frequency after tuning */
#define F_IN            F_FRC           /* FRC as input for PLL */
#define F_OSC           F_IN*40/(2*2)   /* fOSC = fIN*M/(N1*N2): (PLL) oscillator frequency */
#define F_CY            F_OSC/2         /* instruction cycle frequency */

#define BAUDRATE        115200          /* baudrate for serial interface */
#define BRGVAL          ((F_CY/BAUDRATE)/16)-1

// CONFIG4
#pragma config DSWDTPS = DSWDTPSF       // DSWDT Postscale Select (1:2,147,483,648 (25.7 days))
#pragma config DSWDTOSC = LPRC          // Deep Sleep Watchdog Timer Oscillator Select (DSWDT uses Low Power RC Oscillator (LPRC))
#pragma config RTCOSC = LPRC            // RTCC Reference Oscillator  Select (RTCC uses Low Power RC Oscillator (LPRC))
#pragma config DSBOREN = ON             // Deep Sleep BOR Enable bit (BOR enabled in Deep Sleep)
#pragma config DSWDTEN = OFF            // Deep Sleep Watchdog Timer (DSWDT disabled)

// CONFIG3
#pragma config WPFP = WPFP0             // Write Protection Flash Page Segment Boundary (Page 0 (0x0))
#pragma config SOSCSEL = IO             // Secondary Oscillator Pin Mode Select (SOSC pins have digital I/O functions (RA4, RB4))
#pragma config WUTSEL = LEG             // Voltage Regulator Wake-up Time Select (Default regulator start-up time used)
#pragma config WPDIS = WPEN             // Segment Write Protection Disable (Segmented code protection enabled)
#pragma config WPCFG = WPCFGDIS         // Write Protect Configuration Page Select (Last page and Flash Configuration words are unprotected)
#pragma config WPEND = WPENDMEM         // Segment Write Protection End Page Select (Write Protect from WPFP to the last page of memory)

// CONFIG2
#pragma config POSCMOD = NONE           // Primary Oscillator Select (Primary Oscillator disabled)
#pragma config I2C1SEL = PRI            // I2C1 Pin Select bit (Use default SCL1/SDA1 pins for I2C1 )
#pragma config IOL1WAY = ON             // IOLOCK One-Way Set Enable (Once set, the IOLOCK bit cannot be cleared)
#pragma config OSCIOFNC = OFF           // OSCO Pin Configuration (OSCO pin functions as clock output (CLKO))
#pragma config FCKSM = CSDCMD           // Clock Switching and Fail-Safe Clock Monitor (Sw Disabled, Mon Disabled)
#pragma config FNOSC = FRCPLL           // Initial Oscillator Select (Fast RC Oscillator with Postscaler and PLL module (FRCPLL))
#pragma config PLL96MHZ = ON            // 96MHz PLL Startup Select (96 MHz PLL Startup is enabled automatically on start-up)
#pragma config PLLDIV = NODIV           // USB 96 MHz PLL Prescaler Select (Oscillator input used directly (4 MHz input))
#pragma config IESO = ON                // Internal External Switchover (IESO mode (Two-Speed Start-up) enabled)

// CONFIG1
#pragma config WDTPS = PS32768          // Watchdog Timer Postscaler (1:32,768)
#pragma config FWPSA = PR128            // WDT Prescaler (Prescaler ratio of 1:128)
#pragma config WINDIS = OFF             // Windowed WDT (Standard Watchdog Timer enabled,(Windowed-mode is disabled))
#pragma config FWDTEN = OFF             // Watchdog Timer (Watchdog Timer is disabled)
#pragma config ICS = PGx1               // Emulator Pin Placement Select bits (Emulator functions are shared with PGEC2/PGED2)
#pragma config GWRP = OFF               // General Segment Write Protect (Writes to program memory are allowed)
#pragma config GCP = OFF                // General Segment Code Protect (Code protection is disabled)
#pragma config JTAGEN = OFF             // JTAG Port Enable (JTAG port is disabled)


/*
 * Hardware initialization.
 */
void initHardware(void)
{
    /*------------------------------------------------------------------------*/
    /* Configuration of oscillator */
    /*------------------------------------------------------------------------*/
    /* Configure PLL prescaler, PLL postscaler, PLL divisor
     * FRC frequency: 7.37MHz -> tuning to 8MHz
     * NOTE: FRC frequency is not accurate -> frequency may vary!
     * fCY = 8MHz * 40/(2*2) / 2 = 40MHz
     */

//    PLLFBD = 40;                /* M = 40 */
//    CLKDIVbits.PLLPOST = 0;     /* N2 = 2 */
//    CLKDIVbits.PLLPRE = 0;      /* N1 = 2 */

    /* Switch to FRC with PLL */
    __builtin_write_OSCCONH(0x01);
    __builtin_write_OSCCONL(0x01);
    /* Wait for Clock switch to occur */
    while (OSCCONbits.COSC != 0b001);
    /* Wait for PLL to lock */
    while(OSCCONbits.LOCK != 1) {};

    /* Enable watchdog */
    RCONbits.SWDTEN = 1;


    /*------------------------------------------------------------------------*/
    /* Configuration of serial port */
    /*------------------------------------------------------------------------*/
    RPINR18bits.U1RXR = 11;     /* map UART #1 rx to RP11 */
    RPOR5bits.RP10R = 3;        /* map UART #1 tx to RP43 */

//    U1BRG = BRGVAL;             /* set baudrate */
    // NOTE: automatic baud rate measurement is used due to inaccurate oscillator
    U1MODEbits.ABAUD = 1;       /* enable automatic baud rate measurement */
    U1MODEbits.PDSEL = 0b00;    /* 8-bit data, no parity */
    U1MODEbits.STSEL = 0;       /* 1 stop bit */
    U1MODEbits.UARTEN = 01;     /* enable UART 1 */
    U1STAbits.UTXEN = 1;        /* enable transmitter */


    /*------------------------------------------------------------------------*/
    /* Configuration of IO ports */
    /*------------------------------------------------------------------------*/
    /* LED1 */

    /* LED2 */
    TRISBbits.TRISB13 = 0;  /* digital output */
    LATBbits.LATB13 = 0;    /* LED off */
    /* LED3 */
    TRISBbits.TRISB14 = 0;  /* digital output */
    LATBbits.LATB14 = 0;    /* LED off */
    /* LED4 */
    TRISBbits.TRISB15 = 0;  /* digital output */
    LATBbits.LATB15 = 0;    /* LED off */

    /* UART1 Rx */
    TRISBbits.TRISB11 = 1;  /* RB11: digital input (UART Rx configuration is done in serialInit()) */
    CNPU1bits.CN11PUE = 1;  /* activate weak pull up */
    
    /* VPOT */
    AD1PCFGLbits.PCFG0 = 0; /* AN0: analog input */
    /* VTEMP */
    AD1PCFGLbits.PCFG1 = 0; /* AN1: analog input */

    /* PIEZO */
    TRISBbits.TRISB5 = 0;       /* digital ouput */
    RPOR2bits.RP5R = 0b10010;   /* map OC1 to RP5 */



    

    /*------------------------------------------------------------------------*/
    /* Configuration of ADC */
    /*------------------------------------------------------------------------*/
 //   AD1CON1bits.AD12B = 0;      /* 10bit mode */
    AD1CON2bits.VCFG = 0b000;   /* AVdd and AVss as voltage reference */
    AD1CON3bits.ADRC = 1;       /* ADC uses internal RC clock */
 //   AD1CON2bits.CHPS = 0b00;    /* converts CH0 */
    AD1CHS0bits.CH0SA = 0b0000; /* channel 0 positive input is AN0 */
  //  AD1CON1bits.SIMSAM = 1;     /* simultaneously sampling */
    AD1CON1bits.ASAM = 1;       /* sampling begins after last conversion */
    AD1CON1bits.SSRC = 0b111;   /* internal counter ends sampling and starts conversion (auto conversion) */
    AD1CON3bits.SAMC = 0b00000; /* not used for SSRC = 0b111 */
    AD1CON1bits.FORM = 0b00;    /* integer output (right aligned) */
    AD1CON1bits.ADON = 1;       /* enable ADC */


//    /*------------------------------------------------------------------------*/
//    /* Configuration of timer1 unit for sampling time */
//    /*------------------------------------------------------------------------*/
//    T1CONbits.TCS = 0;      /* internal clock (fCY) */
//    T1CONbits.TGATE = 0;    /* timer in counter mode */
//    T1CONbits.TCKPS = 0b00; /* 1:1 prescaler */
//    PR1 = 4000;             /* @ fCY = 40MHz -> 100�s timebase */
//    TMR1 = 0x0000;          /* clear timer 1 value */
//    IFS0bits.T1IF = 0;      /* clear interrupt flag */
//    IEC0bits.T1IE = 1;      /* set interrupt enable bit */
//    T1CONbits.TON = 1;      /* start timer 1 */


//    /*------------------------------------------------------------------------*/
//    /* Configuration of timer2 unit for compare unit (PWM) */
//    /*------------------------------------------------------------------------*/
//    T2CONbits.TCS = 0;      /* internal clock (fCY) */
//    T2CONbits.TGATE = 0;    /* timer in counter mode */
//    T2CONbits.TCKPS = 0b00; /* 1:1 prescaler */
//    PR2 = PWM_FREQUENCY;    /* @ fCY = 40MHz -> 4kHz PWM frquency */
//    TMR2 = 0x0000;          /* clear timer 2 value */
//    T2CONbits.TON = 1;      /* start timer 2 */

//    /*------------------------------------------------------------------------*/
//    /* Configuration of compare unit for PWM */
//    /*------------------------------------------------------------------------*/
//    OC1CONbits.OCTSEL = 0;      /* timer 2 as clock source */
//    OC1R = 0x0000;              /* clear compare value */
//    OC1RS = 0x0000;             /* clear secondary compare value */
//    OC1CONbits.OCM = 0b110;     /* PWM mode, fault pin disabled */
    
    
        /* Initialize Timer 1
     * Each configuration sets the MCU's oscillator to 32MHz (see the respective
     * "system.c" for configuration bits).  For the LED to blink
     * at a rate of 1Hz, it must be toggled at 2Hz.
     *
     * Calculation:
     * Fcy = Fosc / 2 = 16E6 [Hz]
     * T1CONbits.TCKPS = 3 divides the input clock by 256.
     * PR1 = Fcy[Hz] / 256 / 2[Hz] = 0x7A12;
     */
    _T1IF = 0;
    _T1IE = 1;
    TMR1 = 0x0000;
    PR1 = 0x5000;
    T1CONbits.TCKPS = 0;
    T1CONbits.TON = 1;

}


/******************************************************************************/
/* Initialization of serial interface (overriding methods) */
/******************************************************************************/
void initSerial(tSerial* serial)
{
    serial->send = (void (*)(tInterface*, uint8))sendSerial;
    serial->receive = (uint8 (*)(tInterface*))receiveSerial;
    serial->isReceiveDataAvailable = (uint8 (*)(tInterface*))isReceiveDataAvailable;
    serial->isSendReady = (uint8 (*)(tInterface*))isSendReady;
}

static void sendSerial(tSerial* serial, uint8 data)
{
    U1TXREG = data;
}

static uint8 receiveSerial(tSerial* serial)
{
    if (U1STA & 0x0E)
    {
        U1STAbits.OERR = 0; /* reset error */
        return ((uint8)0);
    }
    return (U1RXREG);
}

static uint8 isReceiveDataAvailable(tSerial* serial)
{
    return (U1STAbits.URXDA);
}

static uint8 isSendReady(tSerial* serial)
{
    return (!U1STAbits.UTXBF);
}

/*
 * Interrupt service routine for timer 1.
 */
void __attribute__((__interrupt__, no_auto_psv)) _T1Interrupt(void)
{
    IFS0bits.T1IF = 0; /* clear interrupt flag */
    mainTask();
}
